   program checkoutput

   use dal
   use errorhandling

   implicit none

   type(DataSetT) :: inDataSetH                                      ! Input dataset handle.
   type(ArrayT) :: inarrayH                                          ! Input array handle.
   type(TableT) :: intableH                                          ! Input table handle.
   type(TableT) :: modesTableH                                       ! Modes table dataset handle.
   type(ColumnT), dimension(2) :: inColumnH                          ! Input column handle.
   type(ColumnT), dimension(3) :: modesColumnH                       ! Modes column handle.
   real(kind=single), dimension(:, :), pointer :: real32ImageP       ! Real array pointer.
   integer(kind=int16), dimension(:), pointer :: rawXP               ! Raw x column pointer.
   integer(kind=int16), dimension(:), pointer :: rawYP               ! Raw y column pointer.
   integer(kind=int32), dimension(:), pointer :: modeP               ! Mode  column pointer.
   integer(kind=int32), dimension(:), pointer :: windowDXP           ! WindowDX column pointer.
   integer(kind=int32), dimension(:), pointer :: windowDYP           ! WindowDY column pointer.

   character(len=255) :: dataMode                                    ! Input file data mode.

   integer(kind=int32), dimension(2) :: imageDim                     ! Image dimensions.
   integer(kind=int32) :: nDim                                       ! Number of Image dimensions.
   integer(kind=int32) :: binX                                       ! Total binning on X axis.
   integer(kind=int32) :: binY                                       ! Total binning on Y axis.
   integer(kind=int32) :: binAx1                                     ! DPU x-axis binning factors.
   integer(kind=int32) :: binAx2                                     ! DPU y-axis binning factors.
   character(len=255) :: binBPE                                      ! bpe binning factor.

   integer(kind=int32) :: rawXMin                                    ! Minimum value for RAWX.
   integer(kind=int32) :: rawXMax                                    ! Maximum value for RAWX.
   integer(kind=int32) :: rawYMin                                    ! Minimum value for RAWY.
   integer(kind=int32) :: rawYMax                                    ! Maximum value for RAWY.

! Read in ODF Image / Event List file.

   inDataSetH = dataSet("testoutput.fts", read)

! Determine file type from file name in primary header.

   dataMode = trim(stringAttribute(inDataSetH, "DATAMODE"))
   binBPE = trim(stringAttribute(inDataSetH, "BINBPE"))

! Read in keywords and data from input file.

   if (dataMode == "IMAGING") then

! OM Imaging Mode Data Image file.

! Get a handle on the image array extension.

      inArrayH = array(inDataSetH, "PRIMARY", read)

! Get the dimensions of the image.

      nDim = numberofDimensions(inarrayH)
      imageDim = dimensions(inArrayH)

      write(*,*) nDim

      if (nDim /=2) call error("notImage", "OMIMAGE is not image")

! Get a pointer to the image.

      real32ImageP => real32Array2Data(inArrayH)

      if (ANY(real32ImageP < 0.)) call error("negs", "Negative values in OMIMAGE")

! Check BINAX1 and BINAX2 values.

      binAx1 = int32Attribute(inDataSetH, "BINAX1")
      binAx2 = int32attribute(inDataSetH, "BINAX2")
      
  write(*,*) binAx1, binAx2

! Event list.

   else if (dataMode == "FAST") then

! OM Fast Mode Event List file.

! Get a handle on the event list binary table extension.

      inTableH = table(inDataSetH, "OMFAE1")

! Get handles and pointers on the event list binary table extension columns.

      inColumnH(1) = column(inTableH, "RAWX", read)
      rawXP => int16data(inColumnH(1))
      rawXMin = int32Attribute(inColumnH(1), "TLMIN")
      rawXMax = int32Attribute(inColumnH(1), "TLMAX")

      inColumnH(2) = column(inTableH, "RAWY", read)
      rawYP => int16data(inColumnH(2))
      rawYMin = int32Attribute(inColumnH(2), "TLMIN")
      rawYMax = int32Attribute(inColumnH(2), "TLMAX")

! Check values.

      if ( (ANY(rawXP < rawXMin)) .or. (ANY(rawXP > rawXmax)) ) &
           & call error("Outofbounds","Out of bounds values in RAWX")

      if ( (ANY(rawYP < rawYMin)) .or. (ANY(rawYP > rawYmax)) ) &
           & call error("Outofbounds","Out of bounds values in RAWY")

   else 
      call fatal("unkownMode","Unknown data mode")
   end if

! Check modes extension.

   modesTableH = table(inDataSetH, "MODES")

   modesColumnH(1) = column(modesTableH, "MODE", read)
   modeP => int32data(modesColumnH(1))
   
   modesColumnH(2) = column(modesTableH, "WINDOWDX", read)
   windowDXP => int32data(modesColumnH(2))
   
   modesColumnH(3) = column(modesTableH, "WINDOWDY", read)
   windowDYP => int32data(modesColumnH(3))

   write(*,*) "Checked Modes Extension"

   if (dataMode == "IMAGING") then
      if (modeP(0) /= 0) call error("wrongMode", "Mode Keyword is inconsistent with dataMode")
   else if (dataMode == "FAST") then
      if (modeP(0) /= 1) call error("wrongMode", "Mode Keyword is inconsistent with dataMode")
   end if

  if (dataMode == "IMAGING") then

      binX = 2**binAx1
      binY = 2**binAx2
      
      write(*,*) binBPE

      if (binBPE == "T") then
         binX = binX * 2
         binY = binY * 2
      endif

      write(*,*) binX, binY, windowDXP(0), windowDYP(0), ImageDim(1), ImageDim(2)

      if( (windowDXP(0)/binX)*(windowDYP(0)/binY) /= ImageDim(1)*ImageDim(2) ) &
    & call error("WrongWin", "Incorrect Window Attributes.")
       
   endif

   if (dataMode == "IMAGING") then
      call release(inArrayH)
   else if (dataMode == "FAST") then
      call release(inColumnH(1))
      call release(inColumnH(2))
      call release(inTableH)
   end if

   call release(modesColumnH(1))
   call release(modesColumnH(2))
   call release(modesColumnH(3))
   call release(modestableH)
   call release(inDataSetH)

end program
